/* ************************************************************************************
 * Das Modul beinhaltet Funktionen zur Berechnung von Voll- und Neumond.
 * Es handelt sich um keine universell einsetzbaren Funktionen. Die Objekte der
 * Ausgabefelder sind in den Funktionen verwendet worden. Sie mssen angepasst
 * oder durch Variablen ersetzt werden.
 * Die Funktionen und Algorithmen haben keinen Anspruch auf
 * hohe Genauigkeit, Vollstndigkeit und Richtigkeit.
 *
 * The module contains functions for calculating full and new moon. 
 * These are not universally applicable functions. The objects of the 
 * output fields have been used in the functions. They must be adjusted 
 * or replaced by variables. 
 * The functions and algorithms have no claim to 
 * high accuracy, completeness and correctness.
 *
 *	Funktionsliste- Function list:
 *	--------------------------------------
 *
 *	- fullMoon()								-	Berechnet das JD des Vollmonds.
 *														Calculates the JD of Full moon
 *  - newMoon()								-	Berechnet das JD des Neumonds.
 *														Calculates the JD of New moon
 *	- sunMoonEventsOccur()			- 	Berechnet ob Mond/Sonnenfinsternis
 *											  			mglich sind
 *														Calculates whether Moon/Solar eclipse
 *											  			are possible.
 * - sunDist()									-	Berechnet die Entfernung Erde-Sonne
 *											  			Calculates the distance earth-sun
 * - magSolarEclipse()					-	Berechnet Sonnenfinsternisse
 *											  			Calculates sun eclipses
 * - magLunarEclipse()					-	Berechnet Mondfinsternisse
 *											  			Calculates moon eclipses
 * - UTFirstContactLunarEclipse()	-	Berechnet die UT des Ersten Kontakts bei Mondfinsternis
 *														Calculates the UT of the first contact at lunar eclipse
 *
 *	Erstellt:								27.11.2020	Dieter Steffan
 *	Update:							11.06.2021	Funktion sunMoonEventsOccur()
 *	Update:							04.07.2021	Funktion sunDist()
 *																Funktion magSolarEclipse()
 *																Funktion magLunarEclipse()
 *																Funktionsbeschreibungen
 *********************************************************************************** */
 
 
 
 
 /* *********************************************************************************
 * Die Funktion berechnet den Abstand zwischen Erde und Sonne in AE
 * (Astronomischen Einheiten). Verwendet wird Zeit, Datum und die 
 * Sommer/Winterzeit (DST) wie auch die Zeitzonenkorrektur (TZ).
 *
 * bergabeparameter:
 *			hour		=	Stunde als Number
 *			minute	=	Minute als Number
 *			second	=	Sekunde als Number
 * 			dst		=	Sommer/Winterzeit (Daylight Savin Time) als Integer
 * 			tz			=	Zeitzone (Timezone) als Integer
 * 			day		=	Tag der Berechnung als Integer
 * 			month	=	Monat der Berechnung als Integer
 * 			year		=	Jahr der Berechnung als Integer
 *
 *	Returnparameter:
 *			RR		=	Entfernung Erde-Sonne in AE
 *	---------------------------------------------------------------------------------------------------
 *  The function calculates the distance between Earth and Sun in AE
 *   (Astronomical Units).  Time, date and the 
 *  Summer /winter time (DST) as well as time zone correction (TZ).
 *
 *  Transfer parameters:
 *			 hour	=	 Hour as Number
 *			 minute	 =	 Minute as number
 *			 second=	 second as number
 * 			 dst		=	 Summer /Wintertime (Daylight Savin Time) as integer
 * 			 tz			=	 Timezone (Timezone) as integer
 * 			 day		=	 Day of calculation as integer
 * 			 month	=	 Month of calculation as integer
 * 			 year		=	 Year of calculation as integer
 *
 *	 Return parameter:
 *			 RR		=	 Distance  Earth-Sun in AE
 *************************************************************************************** */
 function sunDist(hour, minute, second, dst, tz, day, month, year){
  	// bergabeparameter Datentypensicher speichern
	// Transfer parameters Save data type-safe
	var LH = Number(hour);					// Stunde der Berechnung
															// Hour of calculation
	var LM = Number(minute);				// Minute der Berechnung
															// Minute of calculation
	var LS = Number(second);				// Sekunde der Berechnung
															// Second of calculation
	var TZ = parseInt(tz);						// Zeitzone der Berechnung
															// Time zone of calculation
	var DST = parseInt(dst);					// Daylight saving time der Berechnung
															// Daylight saving time of calculation
	var DY = Number(day);					// Tag der Berechnung
															// Day of calculation
	var MN = parseInt(month);				// Monat der Berechnung
															// Month of calculation
	var YR = parseInt(year);					// Jahr der Berechnung
															// Year of calculation
	var GD = 0;										// Greenwich Tag, Monat, Jahr
	var GM = 0;										// Greenwich day, month, year
	var GY = 0;
	var UT = 0.0;									// Weltzeit - Universal Time
	var DJ = 0.0;									// Julianischer Tag - Julian day
	var  T = 0.0;										// Teil des vergangenen Jahrhunderts
															// Part of the last century
	var T2 = 0.0;									// T zum Quadrat
	var RR = 0.0;									// Distanz Erde-Sonne in AE
															// Distance Earth-Sun in AE
	var AT= 0.0;										// Hilfsvariable - Auxilary variable
	var AE = 0.0;									// Exzentrische Anomalie - Excentric anomaly
	var A = 0.0;										// Weitere Hilfsvariablen
	var B = 0.0;										// Further assistance
	var C = 0.0;
	var A1 = 0.0;
	var B1 = 0.0;
	var C1 = 0.0;
	var D1 = 0.0;
	var E1 = 0.0;
	var H1 = 0.0;
	var D2 = 0.0;
	var D3 = 0.0;

	// Den Tag des Berechnungsdatums um 00:00 Uhr berechnen ...
	// Calculate the day of the calculation date at 10:00 . . .
	GD = localtime2GreenwichDay(LH, LM, LS,  DST, TZ, DY, MN, YR);
	// ... und dann Den Tag des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the month of the calculation date at 12:00 . . .
	GM = localtime2GreenwichMonth(LH, LM, LS,  DST, TZ, DY, MN, YR);
	// ... und dann Das Jahr des Berechnungsdatums um 12:00 Uhr berechnen ...
	// ... and than calculate the year of the calculation date at 12:00 . . .
	GY = localtime2GreenwichYear(LH, LM, LS, DST, TZ, DY, MN, YR);
	// ... berechne nun die dezimale Zahl der UT aus der Uhrzeit und Datum mit TZ und DST...
	// . . . now find the decimal number of UTs from the time and date with TZ and DST. . .
	UT = localtime2UT(LH, LM, LS, DST, TZ, DY, MN, YR);
	// ... berechne die Julian. Tage mit Bezug J2000 ...
	// ... calculate the julian days from the start of J2000 ...
	DJ = calDate2JulDate(GD, GM, GY) - 2415020.0;
	// ... den Teil des vergangenen Jahrhunderts berechnen ...
	// . . . calculate the part of the last century . . .
	T = (DJ /  36525.0) + (UT / 876600.0);
	// ... und T zum Quadrat ...
	// ... and T squared ...
	T2 = Math.pow(T, 2);
	// ... berechne die  Hilfsvariablen ...
	// ... calculate the auxilary variables ...
	var A = 100.0021359 * T;
	var B = 360.0 * (A - Math.floor(A));
	var L = 279.69668 + 0.0003025 * T2 + B;
	A = 99.99736042 * T;
	B = 360.0 * (A - Math.floor(A));
	var M1 = 358.47583 - (0.00015 + 0.0000033 * T) * T2 + B;
	var EC = 0.01675104 - 0.0000418 * T - 0.000000126 * T2;
	// ... berechne die Anomaly der Sonne ...
	// ... Calculate the anomaly of the Sun ...
	var AM = M1 / RAD;
	// ... dann die wahre Anomaly der Sonne mit der Exzentrizitt...
	// then the true anomaly of the Sun with the eccentricity ...
	AT = trueAnomaly(AM, EC);
	// ... und die exzentrische Anomalie der Sonne mit Exzentrizitt berechnen ...
	AE = eccentricAnomaly(AM, EC);
	// ... weitere Hilfsvariablen berechnen ...
    A = 62.55209472 * T;
	B = 360.0 * (A - Math.floor(A));
    A1 = deg2Radians(153.23 + B);
    A = 125.1041894 * T;
	B = 360.0 * (A - Math.floor(A));
    B1 = deg2Radians(216.57 + B);
    A = 91.56766028 * T;
	B = 360.0 * (A - Math.floor(A));
    C1 = deg2Radians(312.69 + B);
    A = 1236.853095 * T;
	B = 360.0 * (A - Math.floor(A));
    D1 = deg2Radians(350.74 - 0.00144 * T2 + B);
    E1 = deg2Radians(231.19 + 20.2 * T);
    A = 183.1353208 * T;
	B = 360.0 * (A - Math.floor(A));
    H1 = deg2Radians(353.4 + B);
    D2 = 0.00134 * Math.cos(A1) + 0.00154 * Math.cos(B1) + 0.002 * Math.cos(C1);
    D2 = D2 + 0.00179 * Math.sin(D1) + 0.00178 * Math.sin(E1);
    D3 = 0.00000543 * Math.sin(A1) + 0.00001575 * Math.sin(B1);
    D3 = D3 + 0.00001627 * Math.sin(C1) + 0.00003076 * Math.cos(D1);
    D3 = D3 + 0.00000927 * Math.sin(H1);

    RR = 1.0000002 * (1.0 - EC * Math.cos(AE)) + D3;
	// ... und zurck zum Aufrufer.
	// ... and bach to the caller.
	return (RR);
	
 }// End of sunDist()
 
 
 
 
 /* *********************************************************************************
 * Die Funktion berechnet die Magnitude einer Sonnenfinsternis. Das bedeutet
 * die Strke der Bedeckung durch den Mond. Werte >= 1 sind eine totale
 * Finsternis. Werte < 1 eine teilweise Finsternis.
 *
 * bergabeparameter:
 *			jdNewMoon	=	Das julianische Datum des Neumondes als Number
 *			glong			=	Lngengrad des Beobachtungsortes in dezimalen Grad
 *									und Number
 *			glat				=	Breitengrad des Beobachtungsortes in dezimalen Grad
 *									und Number
 *			occur			=	Ergebnis der Funktion sunMoonEventsOccur() als String
 *
 *	Returnparameter:
 *			MAG			=	Die Magnitude als Dezimalzahl und Number. Wenn
 *									keine Finsternis stattfindet 0.0.
 *									Sonderfall -99 = Es gibt eine Finsternis, aber nicht
 *									an dem Lngen- und Breitengrad.
 *	---------------------------------------------------------------------------------------------------
 *  The function calculates the magnitude of a solar eclipse.  That means
 *  the strength of the occultation by the moon.  Values >= 1 are a total
 *  Darkness .  Values < 1 a partial darkness.
 *
 *  Transfer parameters:
 *			 jdNewMoon	 =	 The Julian date of the new moon as number
 *			 glong			 =	 Longitude of the observation site in decimal degrees
 *             
 *			 glat				 =	 Latitude of the observation site in decimal degrees
 *             
 *			 occur			 =	 Result of the function  sunMoonEventsOccur  () as string
 *
 *	 Return parameter:
 *			 MAG			 =	 The magnitude as decimal number and number.  If
 *									 no darkness takes place 0.0.
 *									 Special case  -99  =  Theres a darkness, but theres no
 *									 at the longitude and latitude.
 *************************************************************************************** */
 function magSolarEclipse(jdNewMoon, glong, glat, occur){								
	var MAG = new Array(0.0, 0.0, 0.0, 0.0);	// Returnwert ist die Magnitude des Events
																	// Return value is the magnitude of the event
	var Pi = 3.141592654;								// Kreiszahl Pi 3,1415.......
																	// 1  x  Number of circles  Pi  3,1415.......
	var TP = Math.PI * 2									// 2 x Kreiszahl Pi 6,28.......
																	// 2  x  Number of circles  Pi  6,28.......
	var DJ = Number(jdNewMoon);					// Julianisches Datum des Vollmondes
																	// Julian date of the full moon
	var arrDate = new Array();						// Array nimmt das Greenwich Datum auf
																	// Array records the Greenwich date
	var GDay = 0.0;										// Greenwich Tag mit Uhrzeit des Vollmondes
																	// Greenwich day and time of the Full Moon
	var IGDay = 0;											// Greenwich Tag des Vollmondes
																	// Greenwich integer day of the Full Moon
	var GMonth = 0;										// Greenwich Monat des Vollmondes
																	// Greenwich month of the Full Moon
	var GYear = 0;											// Greenwich Jahr des Vollmondes
																	// Greenwich Year of the Full Moon
	var XI = 0.0;												// Die Uhrzeit des Greenwich Tages
																	//  The time of Greenwich  Day
	var UTNM = 0.0;										// Universal Time (UT) in dezimalen Stunden
																	// Universal time in decimal hours
	var UT = 0.0;											// Die korregierte dezimale Universal Time (UT) 
																	// The corrected decimal universal time (UT)
	var LY = 0.0;											// Lngengrad der Sonne als Radiant
																	// Longitude of the Sun as Radiant
	var MY = 0.0;											// Lngengrad des Mondes als Radiant
																	// Longitude of the Moon as Radiant
	var BY = 0.0;											// Breitengrad des Mondes als Radiant
																	// Latitude of the Moon as Radiant
	var HY = 0.0;											// Horizontale Parallaxe des Mondes als Radiant
																	// Horizontal Parallax of the Moon as Radiant
	var SB = 0.0;											// Lngengrad der Sonne als Radiant
																	// Longitude of the Sun as Radiant
	var MZ = 0.0;											// Lngengrad des Mondes als Radiant
																	// Longitude of the Moon as Radiant
	var BZ = 0.0;											// Breitengrad des Mondes als Radiant
																	// Latitude of the Moon as Radiant
	var HZ = 0.0;											// Horizontale Parallaxe des Mondes als Radiant
																	// Horizontal Parallax of the Moon as Radiant
	var XH = 0.0;											// Hilfsvariablen fr weitere Berechnungen
	var X0 = 0.0;											// Auxilary variables for further calculations
	var DM = 0.0;
	var LJ = 0.0;
	var Q = 0;
	var MR = 0.0;
	var RR = 0.0;
	var SR = 0.0;
	var X = 0.0;
	var Y = 0.0;
	var SB = 0.0;
	var TM = 0.0;
	var HP = 0.0;
	var P3 = 0.0;
	var ZH = 0.0;
	var TC = 0.0;
	var SH = 0.0;
	var S2 = 0.0;
	var Z2 = 0.0;
	var PS = 0.0;
	var Z1 = -99;
	var H0 = 0.0;
	var RM = 0.0;
	var RN = 0.0;
	var HD = 0.0;
	var RU = 0.0;
	var RP = 0.0;
	var PJ = 0.0;
	var R = 0.0;
	var DD = 0.0;
	var ZD = 0.0;
	var Z6 = -99;
	var Z7 = -99;
	var MG = -99;
	var GLONG = Number(glong);					// Geo. Lngengrad des Beobachters - Geo .  Latitude of the observer
	var GLAT = Number(glat);						// Geo. Breitengrad des Beobachters -  Geo .  Latitude of the observer
	var PAA = 0.0;											// Ekliptische Koordinaten zu quatoriale RA
																	// Ecliptic Coordinates to Equatorial  RA
	var QAA = 0.0;											// Ekliptische Koordinaten zu quatoriale DEC
																	// Ecliptic Coordinates to Equatorial  Dec
	var XAA = 0.0;											// RA zu Stundenwinkeln - RA to hour angle
	var PBB = 0.0;											// Die korregierte Parallaxe des quatorialen Stundenwinkels HA
																	// The corrected parallax of the equatorial hour angle HA
	var QBB = 0.0;											// Die korregierte Parallaxe der quatorialen Deklination
																	// The corrected parallax of the equatorial declination
	var XBB = 0.0;											// Stundenwinkel in RA konvertieren - Hour angle to RA conversion
	// Wenn in dem Monat keine Sonnenfinsternis ist, dann beende sofort wieder ...
	//  If there is no lunar eclipse in that month, then immediately stop ...
	if ( occur == "Keine Sonnenfinsternis" ){
		// Keine Finsterniss und raus und zurck zum Caller.
		//  No darkness and out and back to the caller.
		MG = -99;
		MAG = [MG, Z6, Z1, Z7];
		return(MAG);
	 }
	// Ermittle zuerst aus dem julianischen Datum des Neumondes das gregorianische
	// Datum ...
	// First, find out the Gregorian date of the julian new moon
 	// Date  ...
	arrDate = julianDate2Date(DJ);
	// ... im Array [0] ist der Greenwich Tag + Uhrzeit als Nachkommastellen ...
	// ... in the array [0] is the Greenwich day + time as decimal ...
	GDay = arrDate[0];
	// ... im Array [1] der Greenwich Monat ...
	// ... in the array [1] the Greenwich month ...
	GMonth = arrDate[1];
	// ... im Array [2] das Greenwich Jahr ...
	// ... in the array [2] the Greenwich year ...
	GYear = arrDate[2];
	// ... und hier aus dem kompletten Tag nur die ganze Tageszahl ermitteln ...
	// ...  and here from the complete day only determine the whole number of days ...
	IGDay = Math.floor(GDay);
	// ... nun die dezimale Uhrzeit des Greenwich Tages berechnen ...
	//  ... now calculate the decimal time of the Greenwich day ...
	XI = GDay - IGDay;
	// ... dann aus der dezimalen Uhrzeit des Tages die dezimalen Stunden des Tages
	// berechnen ...
	// ... then from the decimal time of the day the decimal hours of the day
 	// Calculate ...
	UTNM = XI * 24.0;
	// ... UT  um 1 Stunde zur einen Seite verschieben und damit die Berechnungen
	// durchfhren ...
	//  ... UT   move to one side by 1 hour and thus the calculations
 	// execute ...
	UT = UTNM - 1.0;
	//console.log("UTFirstContactSolarEclipse UTNM=" + UTNM);
	// ... und dann den Radianten des Lngengrades der Sonne berechnen ...
	// ...  and then calculate the radiant of the longitude of the Sun ...
	LY = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... und den Radianten des Lngengrades des Mondes berechnen ...
	// ...  and calculate the radiant of the longitude of the Moon ...
	MY = deg2Radians(moonLongitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten des Breitengrad des Mondes berechnen ...
	//  ... also calculate the radiant of the latitude of the Moon ...
	BY = deg2Radians(moonLatitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten der horizontalen Parallaxe des Mondes berechnen ...
	//  ... also calculate the radiant of the horizontal parallax of the Moon ...
	HY = deg2Radians(moonHP(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... und  UT 1 Stunde zur anderen Seite verschieben und damit die Berechnungen
	// noch einmal durchfhren ...
	// ... and UT 1  Move the hour to the other side and with it the calculations
 	//  perform again ...
	UT = UTNM + 1.0;
	// ... und dann den Radianten des Lngengrades der Sonne neu berechnen ...
	// und den vorherig berechneten Wert subtrahieren um die Differenz zu bilden ...
	// ...  and then recalculate the radiant of the longitude of the Sun ...
 	// and subtract the previously calculated value to form the difference ...
	SB = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear)) - LY;
	// ... und dann den Radianten des Lngengrades des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the longitude of the Moon ...
	MZ = deg2Radians(moonLongitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten des Breitengrad des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the latitude of the Moon ...
	BZ = deg2Radians(moonLatitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten der horizontalen Parallaxe des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the horizontal parallax of the Moon ...
	HZ = deg2Radians(moonHP(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... wenn der Lngengrad der Sonne kleiner 0, dann in den positiver Bereich bringen ...
	//  ... if the longitude of the Sun is less than 0, then bring it into the positive range ...
	if (SB < 0) SB = SB + TP;
	// ... Berechnung und Zuweisung vieler Hilfsvariablen ...
	//  ... Calculation and assignment of many auxiliary variables ...
	XH = UTNM;
    X = MY;
	Y = BY;
	TM = XH - 1.0;
	HP = HY;
	// ... springe zur Funktion sub7390() und berechne die ekliptischen Koordinaten
	// fr P=Lngengrad und Q=Breitengrad ...
	//  ... jump to sub7390 () and compile the ecliptic coordinates
 	// for P = Longitude and Q = latitude ...
	sub7390();
	// ... nach Rcksprung hierher besetze einige Variablen neu ...
	//  ... after going back here, fill some variables new ...
	MY = P;
	BY = Q;
	X = MZ;
	Y = BZ;
	TM = XH + 1.0;
	HP = HZ;
	// ... springe wieder zur Funktion sub7390() und berechne die ekliptischen Koordinaten
	// fr P=Lngengrad und Q=Breitengrad ...
	//  ... jump again to sub7390 () and compile the ecliptic coordinates
 	// for P = Longitude and Q = latitude ...
	sub7390();
	// ... nach Rcksprung hierher besetze einige Variablen neu ...
	//  ... after going back here, fill some variables new ...
	MZ = P;
	BZ = Q;
	X0 = XH + 1.0 - (2.0 * BZ / (BZ - BY));
	DM = MZ - MY;
	// ... wenn der Lngengrad des Mondes kleiner 0, dann in den positiver Bereich bringen ...
	//  ... if the longitude of the moon is less than 0, then bring it into the positive range ...
    if (DM < 0) DM = DM + TP;
    LJ = (DM - SB) / 2.0;
	Q = 0;
	MR = MY + (DM * (X0 - XH + 1.0) / 2.0);
    UT = X0 - 0.13851852;
	RR = sunDist(UT, 0, 0, 0, 0, IGDay, GMonth, GYear);
	SR = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	SR = SR + deg2Radians(nutationLong(IGDay, GMonth, GYear)  - 0.00569);
	X = SR;
	Y = 0;
	TM = UT;
	HP = 0.00004263452 / RR;
	// ... springe wieder zur Funktion sub7390() und berechne die ekliptischen Koordinaten
	// fr P=Lngengrad und Q=Breitengrad ...
	//  ... jump again to sub7390 () and compile the ecliptic coordinates
 	// for P = Longitude and Q = latitude ...
	sub7390();
	// ... nach Rcksprung hierher besetze einige Variablen neu ...
	//  ... after going back here, fill some variables new ...
	SR = P;
	BY = BY - Q;
	BZ = BZ - Q;
	P3 = 0.00004263;
	ZH = (SR - MR) / LJ;
	TC = X0 + ZH;
	SH = (((BZ - BY) * (TC - XH - 1.0) / 2.0) + BZ) / LJ;
	S2 = SH * SH;
	Z2 = ZH * ZH;
	PS = P3 / (RR * LJ);
	Z1 = (ZH * Z2 / (Z2 + S2)) + X0;
	H0 = (HY + HZ) / (2.0 * LJ);
	RM = 0.272446 * H0;
	RN = 0.00465242 / (LJ * RR); HD = H0 * 0.99834;
	RU = (HD - RN + PS) * 1.02; RP = (HD + RN + PS) * 1.02;
	PJ = Math.abs(SH * ZH / Math.sqrt(S2 + Z2));
	R = RM + RN; DD = Z1 - X0; DD = DD * DD - ((Z2 - (R * R)) * DD / ZH);
	// ... ein Fehler tritt auf und an diesem Ort ist keine Sonnenfinsternis sichtbar ...
	//  ... an error occurs and in this place  no solar eclipse is visible ...
    if (DD < 0) {
            MG = -99;
			MAG = [MG, Z6, Z1, Z7];
			// ... kehre zum Aufrufer zurck.
			// ... back to the caller.
            return(MAG);
	}
	// ... hier geht es weiter, wenn kein Fehler auftritt ...
	// ... here it continues, if no error occurs ...
	ZD = Math.sqrt(DD); Z6 = Z1 - ZD;
	Z7 = Z1 + ZD - lInt((Z1 + ZD) / 24.0) * 24.0;
	if (Z6 < 0) Z6 = Z6 + 24.0;
	MG = (RM + RN - PJ) / (2.0 * RN);
	// ... Unterfunktion zur Berechnung der ekliptischen Lngen- und Breitengrade ...
	//  ... subfunction for calculating ecliptic longitude and latitude ...
	function sub7390(){
		// ... berechnet aus den ekliptischen Koordinaten und Greenwich Datum die quatoriale Rektaszension RA ...
		PAA = eclipticCoords2EquatorialRA(X * RAD, 0, 0,Y * RAD, 0, 0, IGDay, GMonth, GYear);
		// ... berechnet aus den ekliptischen Koordinaten und Greenwich Datum die quatoriale Deklination Dec ...
		QAA = eclipticCoords2EquatorialDec(X * RAD, 0, 0,Y * RAD, 0, 0, IGDay, GMonth, GYear);
		// ... berechnet aus der quatorialen RA, dem Greenwich Datum und dem Lngengrad den Stundenwinkel
		// in dezimalen Grad ...
		XAA = ra2HourAngle(PAA/15.0, 0, 0, TM,0, 0, 0, 0, IGDay, GMonth, GYear, GLONG);
		// ... berechnet aus dem Stundenwinkel, der quatorialen Deklination, dem Breitengrad und der horizontalen
		// Parallaxe die Parallaxe der RA
		PBB = parallaxHA(XAA, 0, 0, QAA, 0, 0, "True", GLAT, 0, (HP * RAD));
		// ... berechnet aus dem Stundenwinkel, der quatorialen Deklination, dem Breitengrad und der horizontalen
		// Parallaxe die Parallaxe der Dec
		QBB = parallaxDec(XAA, 0, 0, QAA, 0, 0, "True", GLAT, 0, (HP * RAD));
		// ... berechnet aus der parallaxe der RA, dem Greenwich Datum und dem Lngengrad  die RA
		XBB =  angleHours2RA(PBB, 0, 0, TM, 0, 0, 0, 0, IGDay, GMonth, GYear, GLONG);
		// ... berechnet den ekliptischen Lngengrad aus den quatorialen Koordinaten Parallaxe  RA + Dec und
		// dem Greenwich Datum ...
		P = equatorial2EclipticLong(XBB, 0, 0, QBB, 0, 0, IGDay, GMonth, GYear) / RAD;
		// ... errechnet den ekliptischen Breitengrad aus den quatorialen Koordinaten Parallaxe der RA + Dec und
		// dem Greenwich Datum ...
		Q = equatorial2EclipticLat(XBB, 0, 0, QBB, 0, 0, IGDay, GMonth, GYear) / RAD;
		return;
	}
	//	Ergebnis - Result
	//	MAG = MG;
	//	0.0 oder -99 oder die Magnitude
	// Result  Result
	// MAG = MG;
	// 0.0 or -99 or the magnitude
	MAG[0] = MG;
	// Die UT des ersten Kontakts der Sonnenfinsternis
	// The UT of the first contact of the solar eclipse
	MAG[1] = Z6;
	// UT Max Kontakt, also die Mitte der Sonnenfinsternis
	// UT Max contact, i.e. the center of the solar eclipse
	MAG[2] = Z1;
	// UT letzter Kontakt, also das Ende der Sonnenfinsternis
	// UT last contact, i.e. the end of the solar eclipse
	MAG[3] = Z7;
	 // ... und zurck zum Aufrufer.
	 // ... and back to the caller.
	 return(MAG);
	
 }// End of magSolarEclipse()
 
 
 
 
 /* *********************************************************************************
 * Die Funktion berechnet die Universalzeit (UT) des ersten Kontaktes bei einer
 * Mondfinsternis. Das ist der Beginn der penumbralen Phase, also der Eintritt
 * des Mondes in den Halbschatten der Erde.
 *
 * bergabeparameter:
 *			jdFullMoon	=	Julianisches Datum des Vollmondes als Number
 *			ds					=	Daylight Savings Time (DST) Sommer/Winterzeit als Integer
 *			tz					=	Zeitzone (TZ) als Integer
 *			occur			=	Ergebnis der Funktion sunMoonEventsOccur() als Sting
 *
 *	Returnparameter:
 *			Z6		=	UT als Dezimalzahl und Number
 *	---------------------------------------------------------------------------------------------------
 *  The function calculates the universal time (UT) of the first contact at a
 *  Lunar eclipse .  This is the beginning of the penumbral phase, i.e. the entrance
 *  of the moon in the half-shadow of the earth.
 *
 *  Transfer parameters:
 *			 jdFullMoon	 =	 Julian date of the full moon as number
 *			 ds					 =	 Daylight Savings Time (DST) Summer /Wintertime as integer
 *			 tz					 =	 Time zone (TZ) as integer
 *			 occur			 =	 Result of the function  sunMoonEventsOccur  () as a sting
 *
 *	 Return parameter:
 *			 Z6		 =	 UT as decimal number and number
 *************************************************************************************** */
 function UTFirstContactLunarEclipse(jdFullMoon, ds, tz, occur){
	var ZC = parseInt(tz);				// Zeitzonen Korrektur (TZ)
													// Time zone correction (TZ)
	var DJ = Number(jdFullMoon);	// Julianisches Datum des Vollmondes
													// Julian date of the full moon
	var DS = parseInt(ds);				// Sommer/Winterzeit Korrektur
													// Daylight savings time correction
	var OCCUR = String(occur);		// Ergebnis der Funktion sunMoonEventsOccur()
													// Result of the function sunMoonEventsOccur()
	var Pi = 3.141592654;				// Kreiszahl Pi 3,1415.......
													// 1  x  Number of circles  Pi  3,1415.......
	var TP = Pi * 2.0;						// 2 x Kreiszahl Pi 6,28.......
													// 2  x  Number of circles  Pi  6,28.......
	var Z6 = 0.0;							// Dezimale Uhrzeit des ersten Kontakts und Start penumbrale Phase
													// Array records the Greenwich date
	var GDay = 0.0;						// Greenwich Tag mit Uhrzeit des Vollmondes
													// Greenwich day and time of the Full Moon
	var IGDay = 0;							// Greenwich Tag des Vollmondes
													// Greenwich integer day of the Full Moon
	var GMonth = 0;						// Greenwich Monat des Vollmondes
													// Greenwich month of the Full Moon
	var GYear = 0;							// Greenwich Jahr des Vollmondes
													// Greenwich Year of the Full Moon
	var XI = 0.0;								// Die Uhrzeit des Greenwich Tages
													//  The time of Greenwich  Day
	var UTFM = 0.0;						// Universal Time (UT) in dezimalen Stunden
													// Universal time in decimal hours
	var UT = 0.0;							// Die korregierte dezimale Universal Time (UT) 
													// The corrected decimal universal time (UT)
	var LY = 0.0;							// Lngengrad der Sonne als Radiant
													// Longitude of the Sun as Radiant
	var MY = 0.0;							// Lngengrad des Mondes als Radiant
													// Longitude of the Moon as Radiant
	var BY = 0.0;							// Breitengrad des Mondes als Radiant
													// Latitude of the Moon as Radiant
	var HY = 0.0;							// Horizontale Parallaxe des Mondes als Radiant
													// Horizontal Parallax of the Moon as Radiant
	var SB = 0.0;							// Lngengrad der Sonne als Radiant
													// Longitude of the Sun as Radiant
	var MZ = 0.0;							// Lngengrad des Mondes als Radiant
													// Longitude of the Moon as Radiant
	var BZ = 0.0;							// Breitengrad des Mondes als Radiant
													// Latitude of the Moon as Radiant
	var HZ = 0.0;							// Horizontale Parallaxe des Mondes als Radiant
													// Horizontal Parallax of the Moon as Radiant
	var XH = 0.0;							// Hilfsvariablen fr weitere Berechnungen
	var X0 = 0.0;							// Auxilary variables for further calculations
	var DM = 0.0;
	var LJ = 0.0;
	var Q = 0;
	var MR = 0.0;
	var RR = 0.0;
	var SR = 0.0;
	var P3 = 0.0;
	var ZH = 0.0;
	var TC = 0.0;
	var SH = 0.0;
	var S2 = 0.0;
	var Z2 = 0.0;
	var PS = 0.0;
	var Z1 = 0.0;
	var H0 = 0.0;
	var RM = 0.0;
	var RN = 0.0;
	var HD = 0.0;
	var RU = 0.0;
	var RP = 0.0;
	var PJ = 0.0;
	var R = 0.0;
	var DD = 0.0;
	var ZD = 0.0;
	var DP = 0.0;
	
	//console.log("UTFirstContactLunarEclipse DJ=" + DJ);
	// Wenn in dem Monat keine Sonnenfinsternis ist, dann beende sofort wieder ...
	//  If there is no lunar eclipse in that month, then immediately stop ...
	if ( OCCUR == "Keine Mondfinsternis" ){
		// Keine Finsterniss und raus und zurck zum Caller.
		//  No darkness and out and back to the caller.
		Z6 = -99;
		return(Z6);
	 }
	// Ermittle zuerst aus dem julianischen Datum des Neumondes das gregorianische
	// Datum ...
	// First, find out the Gregorian date of the julian new moon
 	// Date  ...
	arrDate = julianDate2Date(DJ);
	// ... im Array [0] ist der Greenwich Tag + Uhrzeit als Nachkommastellen ...
	// ... in the array [0] is the Greenwich day + time as decimal ...
	GDay = arrDate[0];
	// ... im Array [1] der Greenwich Monat ...
	// ... in the array [1] the Greenwich month ...
	GMonth = arrDate[1];
	// ... im Array [2] das Greenwich Jahr ...
	// ... in the array [2] the Greenwich year ...
	GYear = arrDate[2];
	// ... und hier aus dem kompletten Tag nur die ganze Tageszahl ermitteln ...
	// ...  and here from the complete day only determine the whole number of days ...
	IGDay = Math.floor(GDay);
	// ... nun die dezimale Uhrzeit des Greenwich Tages berechnen ...
	//  ... now calculate the decimal time of the Greenwich day ...
	XI = GDay - IGDay;
	// ... dann aus der dezimalen Uhrzeit des Tages die dezimalen Stunden des Tages
	// berechnen ...
	// ... then from the decimal time of the day the decimal hours of the day
 	// Calculate ...
	UTFM = XI * 24.0;
	// ... UT  um 1 Stunde zur einen Seite verschieben und damit die Berechnungen
	// durchfhren ...
	//  ... UT   move to one side by 1 hour and thus the calculations
 	// execute ...
	UT = UTFM - 1.0;
	// ... und dann den Radianten des Lngengrades der Sonne berechnen ...
	// ...  and then calculate the radiant of the longitude of the Sun ...
	LY = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... und den Radianten des Lngengrades des Mondes berechnen ...
	// ...  and calculate the radiant of the longitude of the Moon ...
	MY = deg2Radians(moonLongitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten des Breitengrad des Mondes berechnen ...
	//  ... also calculate the radiant of the latitude of the Moon ...
	BY = deg2Radians(moonLatitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten der horizontalen Parallaxe des Mondes berechnen ...
	//  ... also calculate the radiant of the horizontal parallax of the Moon ...
	HY = deg2Radians(moonHP(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... und  UT 1 Stunde zur anderen Seite verschieben und damit die Berechnungen
	// noch einmal durchfhren ...
	// ... and UT 1  Move the hour to the other side and with it the calculations
 	//  perform again ...
	UT = UTFM + 1.0;
	// ... und dann den Radianten des Lngengrades der Sonne neu berechnen ...
	// und den vorherig berechneten Wert subtrahieren um die Differenz zu bilden ...
	// ...  and then recalculate the radiant of the longitude of the Sun ...
 	// and subtract the previously calculated value to form the difference ...
	SB = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear)) - LY;
	// ... und dann den Radianten des Lngengrades des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the longitude of the Moon ...
	MZ = deg2Radians(moonLongitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten des Breitengrad des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the latitude of the Moon ...
	BZ = deg2Radians(moonLatitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten der horizontalen Parallaxe des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the horizontal parallax of the Moon ...
	HZ = deg2Radians(moonHP(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... wenn der Lngengrad der Sonne kleiner 0, dann in den positiver Bereich bringen ...
	//  ... if the longitude of the Sun is less than 0, then bring it into the positive range ...
	if (SB < 0) SB = SB + TP;
	// ... Berechnung und Zuweisung vieler Hilfsvariablen ...
	//  ... Calculation and assignment of many auxiliary variables ...
	XH = UTFM;
	X0 = XH + 1.0 - (2.0 * BZ / (BZ - BY));
	DM = MZ - MY;
	// ... wenn der Lngengrad des Mondes kleiner 0, dann in den positiver Bereich bringen ...
	//  ... if the longitude of the moon is less than 0, then bring it into the positive range ...
    if (DM < 0) DM = DM + TP;
    LJ = (DM - SB) / 2.0;
	Q = 0;
	MR = MY + (DM * (X0 - XH + 1.0) / 2.0);
    UT = X0 - 0.13851852;
	RR = sunDist(UT, 0, 0, 0, 0, IGDay, GMonth, GYear);
	SR = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	SR = SR + deg2Radians(nutationLong(IGDay, GMonth, GYear)  - 0.00569);
	SR = SR + Pi - lInt((SR + Pi) / TP) * TP;
	BY = BY - Q;
	BZ = BZ - Q;
	P3 = 0.00004263;
	ZH = (SR - MR) / LJ;
	TC = X0 + ZH;
	SH = (((BZ - BY) * (TC - XH - 1.0) / 2.0) + BZ) / LJ;
	S2 = SH * SH;
	Z2 = ZH * ZH;
	PS = P3 / (RR * LJ);
	Z1 = (ZH * Z2 / (Z2 + S2)) + X0;
	H0 = (HY + HZ) / (2.0 * LJ);
	RM = 0.272446 * H0;
	RN = 0.00465242 / (LJ * RR); HD = H0 * 0.99834;
	RU = (HD - RN + PS) * 1.02; RP = (HD + RN + PS) * 1.02;
	PJ = Math.abs(SH * ZH / Math.sqrt(S2 + Z2));
	R = RM + RP; DD = Z1 - X0; DD = DD * DD - ((Z2 - (R * R)) * DD / ZH);
    if (DD < 0) {
            Z6 = -99;
            return(Z6);
	}
	ZD = Math.sqrt(DD);
	Z6 = Z1 - ZD; 
    if (Z6 < 0) Z6 = Z6 + 24.0;
	
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(Z6);
	 
 }// End of UTFirstContactLunarEclipse()
 
 
 
 
 // Testfunktion, nicht verwendet.
 // Test function, not used.
 /*function magLunarEclipse1(jdFullMoon, tz, occur){
	var MAG = 0.0;						// Returnwert ist die Magnitude des Events
													// Return value is the magnitude of the event
	var Pi = 3.141592654;				// Kreiszahl Pi 3,1415.......
													// 1  x  Number of circles  Pi  3,1415.......
	var TP = Math.PI * 2					// 2 x Kreiszahl Pi 6,28.......
													// 2  x  Number of circles  Pi  6,28.......
	var TZ = parseInt(tz);				// Zeitzonen Korrektur (TZ)
													// Time zone correction (TZ)
	var DJ = Number(jdFullMoon);	// Julianisches Datum des Vollmondes
													// Julian date of the full moon
	var arrDate = new Array();		// Array nimmt das Greenwich Datum auf
													// Array records the Greenwich date
	var GDay = 0.0;						// Greenwich Tag mit Uhrzeit des Vollmondes
													// Greenwich day and time of the Full Moon
	var IGDay = 0;							// Greenwich Tag des Vollmondes
													// Greenwich integer day of the Full Moon
	var GMonth = 0;						// Greenwich Monat des Vollmondes
													// Greenwich month of the Full Moon
	var GYear = 0;							// Greenwich Jahr des Vollmondes
													// Greenwich Year of the Full Moon
	var XI = 0.0;								// Die Uhrzeit des Greenwich Tages
													//  The time of Greenwich  Day
	var UTFM = 0.0;						// Universal Time (UT) in dezimalen Stunden
													// Universal time in decimal hours
	var UT = 0.0;							// Die korregierte dezimale Universal Time (UT) 
													// The corrected decimal universal time (UT)
	var LY = 0.0;							// Lngengrad der Sonne als Radiant
													// Longitude of the Sun as Radiant
	var MY = 0.0;							// Lngengrad des Mondes als Radiant
													// Longitude of the Moon as Radiant
	var BY = 0.0;							// Breitengrad des Mondes als Radiant
													// Latitude of the Moon as Radiant
	var HY = 0.0;							// Horizontale Parallaxe des Mondes als Radiant
													// Horizontal Parallax of the Moon as Radiant
	var SB = 0.0;							// Lngengrad der Sonne als Radiant
													// Longitude of the Sun as Radiant
	var MZ = 0.0;							// Lngengrad des Mondes als Radiant
													// Longitude of the Moon as Radiant
	var BZ = 0.0;							// Breitengrad des Mondes als Radiant
													// Latitude of the Moon as Radiant
	var HZ = 0.0;							// Horizontale Parallaxe des Mondes als Radiant
													// Horizontal Parallax of the Moon as Radiant
	var XH = 0.0;							// Hilfsvariablen fr weitere Berechnungen
	var X0 = 0.0;							// Auxilary variables for further calculations
	var DM = 0.0;
	var LJ = 0.0;
	var Q = 0;
	var MR = 0.0;
	var RR = 0.0;
	var SR = 0.0;
	var P3 = 0.0;
	var ZH = 0.0;
	var TC = 0.0;
	var SH = 0.0;
	var S2 = 0.0;
	var Z2 = 0.0;
	var PS = 0.0;
	var Z1 = 0.0;
	var H0 = 0.0;
	var RM = 0.0;
	var RN = 0.0;
	var HD = 0.0;
	var RU = 0.0;
	var RP = 0.0;
	var PJ = 0.0;
	var R = 0.0;
	var DD = 0.0;
	var ZD = 0.0;
	var Z6 = 0.0;
	var Z7 = 0.0;
	var Z8 = 0.0;
	var Z9 = 0.0;
	var MG = 0.0;
	
	//console.log("DJ=" + DJ);
	// Wenn in dem Monat keine Mondfinsternis ist, dann beende sofort wieder ...
	//  If there is no lunar eclipse in that month, then immediately stop ...
	if ( occur == "Keine Mondfinsternis" ){
		 // Keine Finsterniss und raus und zurck zum Caller.
		 //  No darkness and out and back to the caller.
		return(MAG);
	 }
	// Ermittle zuerst aus dem julianischen Datum des Vollmondes das gregorianische
	// Datum ...
	// First, find out the Gregorian date of the julian full moon
 	// Date  ...
	arrDate = julianDate2Date(DJ);
	// ... im Array [0] ist der Greenwich Tag + Uhrzeit als Nachkommastellen ...
	// ... in the array [0] is the Greenwich day + time as decimal ...
	GDay = arrDate[0];
	// ... im Array [1] der Greenwich Monat ...
	// ... in the array [1] the Greenwich month ...
	GMonth = arrDate[1];
	// ... im Array [2] das Greenwich Jahr ...
	// ... in the array [2] the Greenwich year ...
	GYear = arrDate[2];
	//console.log("GDay=" + GDay);
	//console.log("GMonth=" + GMonth);
	//console.log("GYear=" + GYear);
	// ... und hier aus dem kompletten Tag nur die ganze Tageszahl ermitteln ...
	// ...  and here from the complete day only determine the whole number of days ...
	IGDay = Math.floor(GDay);
	//console.log("IGDay=" + IGDay);
	// ... nun die dezimale Uhrzeit des Greenwich Tages berechnen ...
	//  ... now calculate the decimal time of the Greenwich day ...
	XI = GDay - IGDay;
	//console.log("XI=" + XI);
	// ... dann aus der dezimalen Uhrzeit des Tages die dezimalen Stunden des Tages
	// berechnen ...
	// ... then from the decimal time of the day the decimal hours of the day
 	// Calculate ...
	UTFM = XI * 24.0;
	//console.log("UTFM=" + UTFM);
	// ... UT  um 1 Stunde zur einen Seite verschieben und damit die Berechnungen
	// durchfhren ...
	//  ... UT   move to one side by 1 hour and thus the calculations
 	// execute ...
	UT = UTFM - 1.0;
	//console.log("UT=" + UT);
	// ... und dann den Radianten des Lngengrades der Sonne berechnen ...
	// ...  and then calculate the radiant of the longitude of the Sun ...
	LY = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	//console.log("LY=" + LY);
	// ... und den Radianten des Lngengrades des Mondes berechnen ...
	// ...  and calculate the radiant of the longitude of the Moon ...
	MY = deg2Radians(moonLongitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	//console.log("MY=" + MY);
	// ... auch den Radianten des Breitengrad des Mondes berechnen ...
	//  ... also calculate the radiant of the latitude of the Moon ...
	BY = deg2Radians(moonLatitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	//console.log("BY=" + BY);
	// ... auch den Radianten der horizontalen Parallaxe des Mondes berechnen ...
	//  ... also calculate the radiant of the horizontal parallax of the Moon ...
	HY = deg2Radians(moonHP(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	//console.log("HY=" + HY);
	
	// ... und  UT 1 Stunde zur anderen Seite verschieben und damit die Berechnungen
	// noch einmal durchfhren ...
	// ... and UT 1  Move the hour to the other side and with it the calculations
 	//  perform again ...
	UT = UTFM + 1.0;
	//console.log("UT=" + UT);
	// ... und dann den Radianten des Lngengrades der Sonne neu berechnen ...
	// und den vorherig berechneten Wert subtrahieren um die Differenz zu bilden ...
	// ...  and then recalculate the radiant of the longitude of the Sun ...
 	// and subtract the previously calculated value to form the difference ...
	SB = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear)) - LY;
	//console.log("SB=" + SB);
	// ... und dann den Radianten des Lngengrades des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the longitude of the Moon ...
	MZ = deg2Radians(moonLongitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	//console.log("MZ=" + MZ);
	// ... auch den Radianten des Breitengrad des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the latitude of the Moon ...
	BZ = deg2Radians(moonLatitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	//console.log("BZ=" + BZ);
	// ... auch den Radianten der horizontalen Parallaxe des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the horizontal parallax of the Moon ...
	HZ = deg2Radians(moonHP(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	//console.log("HZ=" + HZ);
	// ... wenn der Lngengrad der Sonne kleiner 0, dann in den positiver Bereich bringen ...
	//  ... if the longitude of the Sun is less than 0, then bring it into the positive range ...
	if (SB < 0) SB = SB + TP;
	//console.log("SB=" + SB);
	
	XH = UTFM;
    X0 = XH + 1.0 - (2.0 * BZ / (BZ - BY));
	DM = MZ - MY;
	//console.log ("X0=" + X0 + " DM=" + DM);
	// ... wenn der Lngengrad des Mondes kleiner 0, dann in den positiver Bereich bringen ...
	//  ... if the longitude of the moon is less than 0, then bring it into the positive range ...
    if (DM < 0) DM = DM + TP;
	//console.log ("DM=" + DM );
    LJ = (DM - SB) / 2.0;
	Q = 0;
    //console.log ("LJ=" + LJ + " Q=" + Q);
	MR = MY + (DM * (X0 - XH + 1.0) / 2.0);
    //console.log ("MR=" + MR);
    UT = X0 - 0.13851852;
    //console.log ("UT=" + UT);
	RR = sunDist(UT, 0, 0, 0, 0, IGDay, GMonth, GYear);
	//console.log ("RR=" + RR);
	SR = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	//console.log ("SR=" + SR);
	SR = SR + deg2Radians(nutationLong(IGDay, GMonth, GYear)  - 0.00569);
	//console.log ("SR=" + SR);
	SR = SR + Pi - lInt((SR + Pi) / TP) * TP;
	//console.log ("SR=" + SR);
	BY = BY - Q;
	BZ = BZ - Q;
	P3 = 0.00004263;
	ZH = (SR - MR) / LJ;
	TC = X0 + ZH;
	//console.log("ZH=" + ZH + " TC=" + TC);
	SH = (((BZ - BY) * (TC - XH - 1.0) / 2.0) + BZ) / LJ;
	//console.log("SH=" + SH);
	S2 = SH * SH;
	Z2 = ZH * ZH;
	PS = P3 / (RR * LJ);
	//console.log ("S2=" + S2 + " Z2=" + Z2 + " PS=" + PS);
	Z1 = (ZH * Z2 / (Z2 + S2)) + X0;
	//console.log ("Z1=" + Z1);
	H0 = (HY + HZ) / (2.0 * LJ);
	RM = 0.272446 * H0;
	//console.log ("H0=" + H0 + " RM=" + RM );
	RN = 0.00465242 / (LJ * RR); HD = H0 * 0.99834;
	//console.log ("RN=" + RN + " HD=" + HD );
	RU = (HD - RN + PS) * 1.02; RP = (HD + RN + PS) * 1.02;
	//console.log ("RU=" + RU + " RP=" + RP );
	PJ = Math.abs(SH * ZH / Math.sqrt(S2 + Z2));
	//console.log ("PJ=" + PJ);
	R = RM + RP; DD = Z1 - X0; DD = DD * DD - ((Z2 - (R * R)) * DD / ZH);
	//console.log ("R=" + R + " DD=" + DD  + " DD=" + DD);
	
    if (DD < 0) {
            MAG = -99;
            return(MAG);
	}
	
	ZD = Math.sqrt(DD); Z6 = Z1 - ZD;
	//console.log ("ZD=" + ZD + " Z6=" + Z6 );
	Z7 = Z1 + ZD - lInt((Z1 + ZD) / 24.0) * 24.0;
	//console.log ("Z7=" + Z7);
	if (Z6 < 0) Z6 = Z6 + 24.0;
	R = RM + RU; DD = Z1 - X0; DD = DD * DD - ((Z2 - (R * R)) * DD / ZH);
	//console.log ("R=" + R + " DD=" + DD  + " DD=" + DD);
	MG = (RM + RP - PJ) / (2.0 * RM);
	//console.log ("MG=" + MG);
	if (DD < 0) {MAG = MG; return(MAG);}
	ZD = Math.sqrt(DD); Z8 = Z1 - ZD;
	//console.log ("ZD=" + ZD + " Z8=" + Z8);
	Z9 = Z1 + ZD - lInt((Z1 + ZD) / 24.0) * 24.0;
	//console.log ("Z9=" +Z9);
	if (Z8 < 0) Z8 = Z8 + 24.0;
	//console.log ("Z8=" +Z8);
	R = RU - RM; DD = Z1 - X0; DD = DD * DD - ((Z2 - (R * R)) * DD / ZH);
	//console.log ("R=" + R + " DD=" + DD  + " DD=" + DD);
	MG = (RM + RU - PJ) / (2.0 * RM);
	//console.log ("MG=" +MG);
	MAG = MG;
	 // ... und zurck zum Aufrufer.
	 // ... and back to the caller.
	 return(MAG);
	
 }// End of magLunarEclipse()*/
 
 
 
 
 /* *********************************************************************************
 * Die Funktion berechnet die Magnitude einer Mondfinsternis. Das bedeutet
 * die Strke der Bedeckung durch den Mond. Werte >= 1 sind eine totale
 * Finsternis. Werte < 1 eine teilweise Finsternis.
 *
 * bergabeparameter:
 *			jdNewMoon	=	Das julianische Datum des Neumondes als Number
 *			occur			=	Ergebnis der Funktion sunMoonEventsOccur() als String
 *
 *	Returnparameter:
 *			MAG			=	Die Magnitude als Dezimalzahl und Number. Wenn
 *									keine Finsternis stattfindet 0.0.
 *									Sonderfall -99 = Es gibt eine Finsternis, aber nicht
 *									an dem Lngen- und Breitengrad.
 *	---------------------------------------------------------------------------------------------------
 *  The function calculates the magnitude of a lunar eclipse.  That means
 *  the strength of the occultation by the sun.  Values >= 1 are a total
 *  Darkness .  Values < 1 a partial darkness.
 *
 *  Transfer parameters:
 *			 jdFullMoon	 =	 The Julian date of the full moon as number
 *			 occur			 =	 Result of the function  sunMoonEventsOccur  () as string
 *
 *	 Return parameter:
 *			 MAG			 =	 The magnitude as decimal number and number.  If
 *									 no darkness takes place 0.0.
 *									 Special case  -99  =  Theres a darkness, but theres no
 *									 at the longitude and latitude.
 *************************************************************************************** */
 function magLunarEclipse(jdFullMoon, tz, occur){
													// Return value is the magnitude of the event
	var MAG = new Array(0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);										
	var Pi = 3.141592654;				// Kreiszahl Pi 3,1415.......
													// 1  x  Number of circles  Pi  3,1415.......
	var TZ = parseInt(tz);				// Zeitzonen Korrektur (TZ)
													// Time zone correction (TZ)
	var TP = Math.PI * 2					// 2 x Kreiszahl Pi 6,28.......
													// 2  x  Number of circles  Pi  6,28.......
	var DJ = Number(jdFullMoon);	// Julianisches Datum des Vollmondes
													// Julian date of the full moon
	var arrDate = new Array();		// Array nimmt das Greenwich Datum auf
													// Array records the Greenwich date
	var GDay = 0.0;						// Greenwich Tag mit Uhrzeit des Vollmondes
													// Greenwich day and time of the Full Moon
	var IGDay = 0;							// Greenwich Tag des Vollmondes
													// Greenwich integer day of the Full Moon
	var GMonth = 0;						// Greenwich Monat des Vollmondes
													// Greenwich month of the Full Moon
	var GYear = 0;							// Greenwich Jahr des Vollmondes
													// Greenwich Year of the Full Moon
	var XI = 0.0;								// Die Uhrzeit des Greenwich Tages
													//  The time of Greenwich  Day
	var UTFM = 0.0;						// Universal Time (UT) in dezimalen Stunden
													// Universal time in decimal hours
	var UT = 0.0;							// Die korregierte dezimale Universal Time (UT) 
													// The corrected decimal universal time (UT)
	var LY = 0.0;							// Lngengrad der Sonne als Radiant
													// Longitude of the Sun as Radiant
	var MY = 0.0;							// Lngengrad des Mondes als Radiant
													// Longitude of the Moon as Radiant
	var BY = 0.0;							// Breitengrad des Mondes als Radiant
													// Latitude of the Moon as Radiant
	var HY = 0.0;							// Horizontale Parallaxe des Mondes als Radiant
													// Horizontal Parallax of the Moon as Radiant
	var SB = 0.0;							// Lngengrad der Sonne als Radiant
													// Longitude of the Sun as Radiant
	var MZ = 0.0;							// Lngengrad des Mondes als Radiant
													// Longitude of the Moon as Radiant
	var BZ = 0.0;							// Breitengrad des Mondes als Radiant
													// Latitude of the Moon as Radiant
	var HZ = 0.0;							// Horizontale Parallaxe des Mondes als Radiant
													// Horizontal Parallax of the Moon as Radiant
	var XH = 0.0;							// Hilfsvariablen fr weitere Berechnungen
	var X0 = 0.0;							// Auxilary variables for further calculations
	var DM = 0.0;
	var LJ = 0.0;
	var Q = 0;
	var MR = 0.0;
	var RR = 0.0;
	var SR = 0.0;
	var P3 = 0.0;
	var ZH = 0.0;
	var TC = 0.0;
	var SH = 0.0;
	var S2 = 0.0;
	var Z2 = 0.0;
	var PS = 0.0;
	var Z1 = 0.0;
	var H0 = 0.0;
	var RM = 0.0;
	var RN = 0.0;
	var HD = 0.0;
	var RU = 0.0;
	var RP = 0.0;
	var PJ = 0.0;
	var R = 0.0;
	var DD = 0.0;
	var ZD = 0.0;
	var Z6 = 0.0;
	var Z7 = 0.0;
	var Z8 = 0.0;
	var Z9 = 0.0;
	var ZB = 0.0;
	var MG = 0.0;
	var ZCC = 0.0;
	// Wenn in dem Monat keine Mondfinsternis ist, dann beende sofort wieder ...
	//  If there is no lunar eclipse in that month, then immediately stop ...
	if ( occur == "Keine Mondfinsternis" ){
		// Keine Finsterniss und raus und zurck zum Caller.
		//  No darkness and out and back to the caller.
		MG = -99;
		MAG = [MG, Z6, Z8, ZCC, Z1, ZB, Z9, Z7];
		// ... und zurck zum Aufrufer.
		// ... and back to the caller.
		return(MAG);
	 }
	// Ermittle zuerst aus dem julianischen Datum des Vollmondes das gregorianische
	// Datum ...
	// First, find out the Gregorian date of the julian full moon
 	// Date  ...
	arrDate = julianDate2Date(DJ);
	// ... im Array [0] ist der Greenwich Tag + Uhrzeit als Nachkommastellen ...
	// ... in the array [0] is the Greenwich day + time as decimal ...
	GDay = arrDate[0];
	// ... im Array [1] der Greenwich Monat ...
	// ... in the array [1] the Greenwich month ...
	GMonth = arrDate[1];
	// ... im Array [2] das Greenwich Jahr ...
	// ... in the array [2] the Greenwich year ...
	GYear = arrDate[2];
	// ... und hier aus dem kompletten Tag nur die ganze Tageszahl ermitteln ...
	// ...  and here from the complete day only determine the whole number of days ...
	IGDay = Math.floor(GDay);
	// ... nun die dezimale Uhrzeit des Greenwich Tages berechnen ...
	//  ... now calculate the decimal time of the Greenwich day ...
	XI = GDay - IGDay;
	// ... dann aus der dezimalen Uhrzeit des Tages die dezimalen Stunden des Tages
	// berechnen ...
	// ... then from the decimal time of the day the decimal hours of the day
 	// Calculate ...
	UTFM = XI * 24.0;
	// ... UT  um 1 Stunde zur einen Seite verschieben und damit die Berechnungen
	// durchfhren ...
	//  ... UT   move to one side by 1 hour and thus the calculations
 	// execute ...
	UT = UTFM - 1.0;
	// ... und dann den Radianten des Lngengrades der Sonne berechnen ...
	// ...  and then calculate the radiant of the longitude of the Sun ...
	LY = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... und den Radianten des Lngengrades des Mondes berechnen ...
	// ...  and calculate the radiant of the longitude of the Moon ...
	MY = deg2Radians(moonLongitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten des Breitengrad des Mondes berechnen ...
	//  ... also calculate the radiant of the latitude of the Moon ...
	BY = deg2Radians(moonLatitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten der horizontalen Parallaxe des Mondes berechnen ...
	//  ... also calculate the radiant of the horizontal parallax of the Moon ...
	HY = deg2Radians(moonHP(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... und  UT 1 Stunde zur anderen Seite verschieben und damit die Berechnungen
	// noch einmal durchfhren ...
	// ... and UT 1  Move the hour to the other side and with it the calculations
 	//  perform again ...
	UT = UTFM + 1.0;
	// ... und dann den Radianten des Lngengrades der Sonne neu berechnen ...
	// und den vorherig berechneten Wert subtrahieren um die Differenz zu bilden ...
	// ...  and then recalculate the radiant of the longitude of the Sun ...
 	// and subtract the previously calculated value to form the difference ...
	SB = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear)) - LY;
	// ... und dann den Radianten des Lngengrades des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the longitude of the Moon ...
	MZ = deg2Radians(moonLongitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten des Breitengrad des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the latitude of the Moon ...
	BZ = deg2Radians(moonLatitude(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... auch den Radianten der horizontalen Parallaxe des Mondes neu berechnen ...
	//  ... also recalculate the radiant of the horizontal parallax of the Moon ...
	HZ = deg2Radians(moonHP(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	// ... wenn der Lngengrad der Sonne kleiner 0, dann in den positiver Bereich bringen ...
	//  ... if the longitude of the Sun is less than 0, then bring it into the positive range ...
	if (SB < 0) SB = SB + TP;
	XH = UTFM;
    X0 = XH + 1.0 - (2.0 * BZ / (BZ - BY));
	DM = MZ - MY;
	// ... wenn der Lngengrad des Mondes kleiner 0, dann in den positiver Bereich bringen ...
	//  ... if the longitude of the moon is less than 0, then bring it into the positive range ...
    if (DM < 0) DM = DM + TP;
    LJ = (DM - SB) / 2.0;
	Q = 0;
	MR = MY + (DM * (X0 - XH + 1.0) / 2.0);
    UT = X0 - 0.13851852;
	RR = sunDist(UT, 0, 0, 0, 0, IGDay, GMonth, GYear);
	SR = deg2Radians(sunLong(UT, 0, 0, 0, 0, IGDay, GMonth, GYear));
	SR = SR + deg2Radians(nutationLong(IGDay, GMonth, GYear)  - 0.00569);
	SR = SR + Pi - lInt((SR + Pi) / TP) * TP;
	BY = BY - Q;
	BZ = BZ - Q;
	P3 = 0.00004263;
	ZH = (SR - MR) / LJ;
	TC = X0 + ZH;
	SH = (((BZ - BY) * (TC - XH - 1.0) / 2.0) + BZ) / LJ;
	S2 = SH * SH;
	Z2 = ZH * ZH;
	PS = P3 / (RR * LJ);
	Z1 = (ZH * Z2 / (Z2 + S2)) + X0;
	H0 = (HY + HZ) / (2.0 * LJ);
	RM = 0.272446 * H0;
	RN = 0.00465242 / (LJ * RR); HD = H0 * 0.99834;
	RU = (HD - RN + PS) * 1.02; RP = (HD + RN + PS) * 1.02;
	PJ = Math.abs(SH * ZH / Math.sqrt(S2 + Z2));
	R = RM + RP; DD = Z1 - X0; DD = DD * DD - ((Z2 - (R * R)) * DD / ZH);
	// ... ein Fehler tritt auf und an diesem Lngen- und Breitengrad ist keine Mongfinsternis sichtbar ...
	//  ... an error occurs and at this longitude and latitude no mong eclipse is visible ...
    if (DD < 0) {
		// Alle Werte im Return Array speichern ...
		// Store all values in result array ...
		MG = -99;
		MAG = [MG, Z6, Z8, ZCC, Z1, ZB, Z9, Z7];
		// ... und zurck zum Aufrufer.
		// ... and back to the caller.
        return(MAG);
	}
	// ... wenn kein Fehler auftritt geht es hier weiter ...
	//  ...  if no error occurs it goes on here ...
	ZD = Math.sqrt(DD); Z6 = Z1 - ZD;
	Z7 = Z1 + ZD - lInt((Z1 + ZD) / 24.0) * 24.0;
	if (Z6 < 0) Z6 = Z6 + 24.0;
	R = RM + RU; DD = Z1 - X0; DD = DD * DD - ((Z2 - (R * R)) * DD / ZH);
	MG = (RM + RP - PJ) / (2.0 * RM);
	// ... ein Fehler tritt auf und an diesem Lngen- und Breitengrad ist keine Mongfinsternis sichtbar ...
	//  ... an error occurs and at this longitude and latitude no mong eclipse is visible ...
	if (DD < 0) {
		// Alle Werte im Return Array speichern ...
		// Store all values in result array ...
		MG = -99;
		MAG = [MG, Z6, Z8, ZCC, Z1, ZB, Z9, Z7];
		///... und zurck zum Aufrufer.
		// ... and back to the caller.
		return(MAG);
	}
	ZD = Math.sqrt(DD); Z8 = Z1 - ZD;
	Z9 = Z1 + ZD - lInt((Z1 + ZD) / 24.0) * 24.0;
	if (Z8 < 0) Z8 = Z8 + 24.0;
	R = RU - RM; DD = Z1 - X0; DD = DD * DD - ((Z2 - (R * R)) * DD / ZH);
	MG = (RM + RU - PJ) / (2.0 * RM);
	if (DD < 0) {
		// Alle Werte im Return Array speichern ...
		// Store all values in result array ...
		MG = -99;
		MAG = [MG, Z6, Z8, ZCC, Z1, ZB, Z9, Z7];
		//... und zurck zum Aufrufer.
		// ... and back to the caller.
		return(MAG);
	}
    ZD = Math.sqrt(DD);
	// Start der totalen Phase
	// End totale phase
    ZB = Z1 + ZD - lInt((Z1 + ZD) / 24.0) * 24.0;
	ZCC = Z1 - ZD;
	// Start Total Lunar Eclipse
    if (ZCC < 0) ZCC = ZC + 24.0;
	// Magnitude im Index 0
	// Magnitude im Index 0
	MAG[0] = MG;
	// Start Penumbrale Phase =  Index 1
	// Start Penumbralphase = Index 1
	MAG[1] = Z6;
	// Start Umbrale Phase =  Index 2
	// Start umbralphase = Index 2
	MAG[2] = Z8;
	// Start Total Phase =  Index 3
	// Start totalphase = Index 3
	MAG[3] = ZCC;
	// Mitte Total Phase =  Index 4
	// Start middle totalphase = Index 4
	MAG[4] = Z1;
	// Ende Total Phase =  Index 5
	// End totallphase = Index 5
	MAG[5] = ZB;
	// Ende umbrale Phase = Index 6
	// End umbralphase = Index 6
	MAG[6] = Z9;
	// Ende penumbrale Phase = Index 7
	// End penumbralphase = Index 7
	MAG[7] = Z7;
	 // ... und zurck zum Aufrufer.
	 // ... and back to the caller.
	 return(MAG);
	
 }// End of magLunarEclipse()
 
 
 
 /* *********************************************************************************
 * Die Funktion berechnet das Datum und das Vorkommen einer Mond- oder
 * Sonnenfinsternis und gibt den Staus als Zeichenkette an den Aufrufer zurck.
 * Benutzt werden das Berechnungsdatum, Tag im Monat, die Sommer/Winterzeit
 * und die Zeitzone sowie was berechnet werden soll,
 *
 * bergabeparameter:
 * 			dst		=	Sommer/Winterzeit (Daylight Savin Time) als Integer
 * 			tz			=	Zeitzone (Timezone) als Integer
 * 			day		=	Tag der Berechnung als Integer
 * 			month	=	Monat der Berechnung als Integer
 * 			year		=	Jahr der Berechnung als Integer
 *			what		=	Was berechnet werden soll Mond/Sonne als String und
 *							"moon" oder "sun"
 *
 *	Returnparameter:
 *						= Ergebnis String fr Mond/Sonnenfinsternis:
 *							"Mondfinsternis sicher"
 *							"Mondfinsternis mglich"
 *							"Keine Mondfinsternis"
 *							"Sonnenfinsternis sicher"
 *							"Sonnenfinsternis mglich"
 *							"Keine Sonnenfinsternis"
 *	---------------------------------------------------------------------------------------------------
 *  The function calculates the date and occurrence of a moon or moon
 *  Solar eclipse and returns the traffic jam as a string to the caller.
 *  The calculation date, day in month, summer/winter time are used
 *  and the time zone and what to calculate ,
 *
 *  Transfer parameters:
 * 			 dst		=	 Summer /Wintertime (Daylight Savin Time) as integer
 * 			 tz			=	 Timezone (Timezone) as integer
 * 			 day		=	 Day of calculation as integer
 * 			 month	=	 Month of calculation as integer
 * 			 year		=	 Year of calculation as integer
 *			 what	=	 What should be calculated Moon/Sun as string and
 *							 	moon or sun
 *
 *	 Return parameter:
 *							=  Result  String for Moon /Solar eclipse:
 *							 	Lunar eclipse certain
 *							 	Lunar eclipse possible"
 *								"No lunar eclipse"
 *							 	Solar eclipse certain
 *							 	Solar eclipse possible "
 *								 No solar eclipse
 *************************************************************************************** */
 function sunMoonEventsOccur(dst, tz, day, month,year, what){
	var DST = parseInt(dst);						// bergabeparameter in richtigen  Datentypen wandeln
																// Convert transfer parameters to correct data types
	var TZ = parseInt(tz);							// Die Zeitzone - The timezone
	var DAY = parseInt(day);						// Der Tag der Berechnung - The day of the calculation
	var MONTH = parseInt(month); 			// Der Monat der Berechnung - The month of the calculation
	var YEAR = parseInt(year);					// Das Jahr der Berechnung- The year of the calculation
	var WHAT = what.toLowerCase();		// Sonnen- oder Mondfinsternis zu Kleinbuchstaben
	var OCCUR = "";									// Returnwert - Return value
	var D0 = 0;											// Tag, Monat Jahr des Berechnungsdatums
	var M0 = 0;											// Day, month, year of the calculation date
	var J0 = 0.0;											// Julianisches Datum 0.1. Berechnungsjahr
																// Julian date of the 0.1. calculation date
	var DJ = 0.0;										// Julianisches Datum des Berechnungsdatums
																// Julian date of the calculation date
	var  K = 0.0;											// Ab hier Hilfsvariablen der Berechnung
																// From here auxiliary variables of the calculation
	var TN = 0.0; var TF = 0.0; var T= 0.0; var T2 = 0.0; var  E = 0.0;
	var c = 0.0; var B = 0.0; var A = 0.0; var A1 = 0.0; var A2 = 0.0;
	var F = 0.0; var DD = 0.0; var NI = 0.0; var NF = 0.0; var B1 = 0.0;
	var NB = 0.0; var FI = 0.0; var FF = 0.0; var FB = 0.0; var DF = 0.0;
	// Berechne in Greenwich Tag, Monat und Jahr aus dem Berechnungsdatum um
	// 12:00 Uhr Mittags ...
	// Calculate day, month and year in Greenwich from the calculation date
 	// 12:00  noon  ...
	D0 = localtime2GreenwichDay(12, 0, 0, DST,  TZ, DAY, MONTH, YEAR);
	M0 = localtime2GreenwichMonth(12, 0, 0, DST,  TZ, DAY, MONTH, YEAR);
	Y0 = localtime2GreenwichYear(12, 0, 0, DST,  TZ, DAY, MONTH, YEAR);
	// Wenn das Jahr negativ wird + 1. Es muss das Jahr nur immer um
	// 1 Jahr grer werden.
	// If the year becomes negative + 1. It just always has to be around the year
	// grow 1 year.
	if(Y0 < 0)Y0 += 1;	
	// ... und nun das Julianische Datum am 0. Januar Greenwich Berechnungsjahr ...
	// ...  and now the Julian date on January 0 Greenwich calculation year...
	J0 = calDate2JulDate(0, 1, Y0);
	//console.log("lunarEclipseOccur J0=" + J0);
	// ... und dann das Julianische Datum am Greenwich Datum ...
	// ...  and then the Julian date on the Greenwich date...
	DJ = calDate2JulDate(D0, M0, Y0);

	// **************** Hilfsvariablen der Berechnung / Auxiliary variables of the calculation *******************
	K = ((Y0 - 1900.0 + ((DJ - J0)  * 1 / 365) ) * 12.3685);
	K = lInt(K + 0.5);
	TN = K / 1236.85;
	TF = (K + 0.5) / 1236.85;
	T = TN;
	// Zur Unterfunktion
	// To sub function
	sechs855();
	NI = A;
	NF = B;
	NB = F;
	T = TF;
	K = K + 0.5;
	// Zur Unterfunktion
	// To the subfunction
	sechs855();
	FI = A;
	FF = B;
	FB = F;
	// Zur Endefunktion
	// To end function
	sub6940();
	// Unterfunktion 6855
	// Sub function 6855
	function sechs855(){
		T2 = T * T;
		E = 29.53 * K;
		c = 166.56 + (132.87 - 0.009173 * T) * T;
		c = deg2Radians(c);
		B = 0.00058868 * K + (0.0001178 - 0.000000155 * T) * T2;
		B = B + 0.00033 * Math.sin(c ) + 0.75933;
		A = K / 12.36886;
		A1 = 359.2242 + 360.0 * decPart(A ) - (0.0000333 + 0.00000347 * T) * T2;
		A2 = 306.0253 + 360 * decPart(K / 0.9330851);
		A2 = A2 + (0.0107306 + 0.00001236 * T) * T2;
		A = K / 0.9214926;
		F = 21.2964 + 360.0 * decPart(A ) - (0.0016528 + 0.00000239 * T) * T2;
		A1 = unwindDeg(A1);
		A2 = unwindDeg(A2);
		F = unwindDeg(F);
		A1 = deg2Radians(A1);
		A2 = deg2Radians(A2);
		F = deg2Radians(F);
		DD = (0.1734 - 0.000393 * T) * Math.sin(A1 ) + 0.0021 * Math.sin(2 * A1);
		DD = DD - 0.4068 * Math.sin(A2 ) + 0.0161 * Math.sin(2 * A2) - 0.0004 * Math.sin(3 * A2 );
		DD = DD + 0.0104 * Math.sin((2 * F)) - 0.0051 * Math.sin((A1 + A2));
		DD = DD - 0.0074 * Math.sin(A1 - A2) + 0.0004 * Math.sin(2 * F + A1);
		DD = DD - 0.0004 * Math.sin(2 * F - A1 ) - 0.0006 * Math.sin(2 * F + A2) + 0.001 * Math.sin(2 * F - A2);
		DD = DD + 0.0005 * Math.sin(A1 + 2 * A2);
		E1 = Math.floor(E);
		B = B + DD + (E - E1);
		B1 = Math.floor(B);
		A = E1 + B1;
		B = B - B1;
	}// End of sechs855()
	
	// Endfunktion der Hauptfunktion sunMoonEventsOccur()
	// Final function of the main function  sunMoonEventsOccur ()
	function sub6940(){
		// Bedingungen der Mondfinsternisse
		// Conditions of lunar eclipse
		if (WHAT == "moon"){
			// Endscheidungsvariable fr die Mondfinsternis.
			// Final decision variable for the lunar eclipse.
			DF = Math.abs(FB - 3.141592654 * lInt(FB / 3.141592654));
			if  (DF > 0.37) DF = 3.141592654 - DF;
			// Es tritt sicher eine Mondfinsternis auf, weil der Winkel
			// kleiner als 9 30' ist.
			// There will certainly be a lunar eclipse, because the angle
 			// is less than 9 30'.
			OCCUR = "Mondfinsternis sicher";
			// Hier ist eine Mondfinsternis mglich (Ringfrmig).
			// Here a lunar eclipse is possible (ring-shaped).
			if (DF >= 0.242600766) {
				OCCUR = "Mondfinsternis mglich";
				// Winkel zwischen der Linie der Knoten und der Sonne 
				// grer als 12 15' ist eine Mondfinsternis nicht mehr mglich.
				// Angle between the line of the nodes and the Sun 
 				// greater than 12  15' a lunar eclipse is no longer possible.
				if (DF > 0.37) OCCUR = "Keine Mondfinsternis";
			}
		// Bedingungen der Sonnenfinsternisse
		// Conditions of solar eclipse
		}else if (WHAT == "sun") {
			// Endscheidungsvariable fr die Sonnenfinsternis.
			// Final decision variable for the solar eclipse.
			DF = Math.abs(NB - 3.141592654 * lInt(NB / 3.141592654));
			if  (DF > 0.37) DF = 3.141592654 - DF;
			// Es tritt sicher eine Sonnenfinsternis auf, weil der Winkel
			// kleiner als 15 31' ist.
			// There will certainly be a lunar eclipse, because the angle
 			// is less than 15 31'.
			OCCUR = "Sonnenfinsternis sicher";
			// Hier ist eine Sonnefinsternis mglich (Ringfrmig).
			// Here a solar eclipse is possible (ring-shaped).
			if (DF >= 0.242600766) {
				OCCUR = "Sonnenfinsternis mglich";
				// Winkel zwischen der Linie der Knoten und der Sonne 
				// grer als 12 15' ist eine Sonnenfinsternis nicht mehr mglich.
				if (DF > 0.37) OCCUR = "Keine Sonnenfinsternis";
			}
		} 
	}
	// und zurck zum Aufrufer.
	// ... and back to the caller.
	return(OCCUR);
	 
 }// End of sunMoonEventsOccur()
 
 
 
/* *********************************************************************************
 * Die Funktion berechnet das Datum und die lokale Uhrzeit des Neumonds
 * anhand der bergabeparameter als Julianisches Datum UT. Das UT muss
 * danach noch weiter nach LT konvertiert werden.
 * Alle Winkelfunktionen wie Math.sin(Radian); werden hier mit
 * Werten als Radianten versorgt.
 *
 * bergabeparameter:
 * 			dst			=	Sommer/Winterzeit (Daylight Savin Time) als Integer
 * 			tz				=	Zeitzone (Timezone) als Integer
 * 			day			=	Tag der Berechnung als Integer
 * 			month		=	Monat der Berechnung als Integer
 * 			year			=	Jahr der Berechnung als Integer
 *			oJD			=	Julianisches Datum der Berechnung mit Tz und DST zur Kontrolle
 *			oJD0		=	Julianisches Datum der Berechnung 12:00 Uhr Greenwich DST=0, TZ=0
 *
 *	Returnparameter:
 *							= Datum UT als Julianisches Datum als Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the date and local time of the newmoon based on
 * the handover parameters as the Julian date UT. The UT must:
 * then be further converted to LT.
 * All angular functions such as Math. sin (Radian); are available here with
 * Values supplied as radiants.
 *
 * Handover parameters:
 * 			dst 			= summer/winter time (Daylight Savin Time) as integer
 * 			tz 				= Time zone (Timezone) as integer
 * 			day 			= day of calculation as integer
 * 			month 		= month of calculation as integer
 * 			year 			= year of calculation as integer
 *			oJD		 	=	 Julian date of calculation with Tz and DST for control
 *			oJD0	 	=	 Julian date of calculation 12:00 oclock  Greenwich DST=0, TZ=0
 *
 * Returnparameter:
 * 						= Date UT as Julian Date as Number
 *************************************************************************************** */
function newMoon(dst, tz, day, month, year, oJD, oJD0){
	var DST = parseInt(dst);						// bergabeparameter in richtigen  Datentypen wandeln
																// Convert transfer parameters to correct data types
	var TZ = parseInt(tz);							// Die Zeitzone - The timezone
	var DAY = parseInt(day);						// Der Tag der Berechnung - The day of the calculation
	var MONTH = parseInt(month); 			// Der Monat der Berechnung - The month of the calculation
	var YEAR = parseInt(year);					// Das Jahr der Berechnung- The year of the calculation
	var JD = 0.0;										// Das Julianische Datum des Berechnungsdatums
																// The Julian date of the calculation date
	var day0 = 0.0;										// Der Tag des Berechnungsdatums
																// The day of the calculation date
	var month0 = 0.0;									// Der Monat des Berechnungsdatums
																// The month of the calculation date
	var year0 = 0.0;									// Das Jahr des Berechnungsdatums
																// The year of the calculation date
	var J0 = 0.0;											// MJD am 0. Januar Berechnungsjahr 00:00:00.000 TZ=0 DST=0
																// MJD ist 0. Januar year of calculation 00:00:00. 000 TZ=0 DST=0
	var DJ = 0.0;										// MJD am Berechnungsdatum
																// MJD year of calculation
	var  K = 0.0;											// Ab hier Hilfsvariablen der Berechnung
																// From here auxiliary variables of the calculation
	var TN = 0.0; var TF = 0.0; var T= 0.0; var T2 = 0.0; var  E = 0.0;
	var c = 0.0; var B = 0.0; var A = 0.0; var A1 = 0.0; var A2 = 0.0;
	var F = 0.0; var DD = 0.0; var NI = 0.0; var NF = 0.0; var B1 = 0.0;
	var NB = 0.0; var FI = 0.0; var FF = 0.0; var FB = 0.0; var newMoon = 0.0;
	/*
		Hier wird das Eingabedatum einmal, ber das Julianische Datum, hin-
		und her gerechnet. Dies ist notwendig, da das Eingabedatum der Berechnung
		eine Differenz zum Startdatum der Berechnung aufweisen kann. Dies wird durch die
		Parameter Sommer/Winterzeit (DST) und Zeitzone (TZ) hervorgerufen.
		Die Differenzen sind besonders relevant bei Monats- und Jahresbergngen, da das
		Berechnungsdatum ins Vorjahr oder Vormonat rutschen kann.
								
		Z.B. Eingabedatum 			01.01.2020 DST=0 TZ =-12
		         Berechnungsdatum 	02.02.2020
		-----------------------------------------------------------------------------------------------------------------------------
		Here the input date is once, over the Julian date,
		and reckoned back. This is necessary because the input date of the calculation
		may have a difference to the start date of the calculation. This is due to the
		Parameters summer/winter time (DST) and time zone (TZ). 
		The differences are particularly relevant in the case of monthly and annual transitions, as the
		Calculation date can slide into the previous year or month. 
								
		E. g. input date 					01. 01. 2020 DST=0 TZ=-12
		Calculation date: 					02. 02. 2020
	*/											
	// Julianisches Datum des Berechnungsdatum um 12:00:00.000 Uhr
	// mit Sommer/Winterzeit und Zeitzone berechnen ...
	// Julian date of the calculation date at 12:00:00. 000
	// calculate with summer/winter time and time zone . . .
	JD = calcJD(DAY, MONTH, YEAR, 12,  0, 0, 0, DST,  TZ);
	// Ausgabe des JD, nur zur Kontrolle.
	// Output of the JD, only for control.
	oJD.value = JD;
	// Aktuelles Julianisches Datum mit der Funktion calcJD2Date() und dem
	// Parameter des aktuellen Julianischen Datums	(JD)	in 
	// Datumsformat als Array umwandeln ...
	//
	// Array[0] = Das Datum als Gleitkommazahl. 
	// Dezimalstellen sind die Zeit. 
	// Array[1] = Der Monat des Datums als Ganzzahl. 
	// Array[2] = Das Jahr des Datums als Ganzzahl.
	//
	// Current Julian date iwith the function calcJD2Date () and the
	// Parameters of the current Julian date (JD) in
	// Convert date format as an array . . .
	//
	// Array[0] = The date as floating point number. 
	// Decimal places are the time. 
	// Array[1] = The month of the date as an integer. 
	// Array[2] = The year of the date as an integer. 
	var jdDate = calcJD2Date(JD);
	day0 = Math.floor(jdDate[0]);
	month0 = Math.floor(jdDate[1]);
	year0 = Math.floor(jdDate[2] );							
	// Wenn das Jahr negativ wird + 1. Es muss das Jahr nur immer um
	// 1 Jahr grer werden.
	// If the year becomes negative + 1. It just always has to be around the year
	// grow 1 year.
	if(year0 < 0)year0 += 1;					
	// Julianisches Datum des Startdatums  DST=0 und TZ=0, also UT 12:00 Uhr	
	// Julian date of start without DST and TZ, it's UT 12:00 a clock high noon						
	oJD0.value = calcJD(day0, month0, year0, 12, 0, 0, 0, 0, 0);
	// ---------------------------------------------------------------------------------------------------------------------------
	// JD = 2415020 = Datum 0.1.1900 12:00 Uhr oder 31.12.1899 12:00 Uhr - TZ=0 DST=0
	// MJD am 0. Januar Berechnungsjahr
	J0 = calcJD(0, 1, year0, 0, 0,0,0,0,0,0)- 2415020;
	//console.log("J0=" + J0);
	// MJD am Berechnungsdatum 00:00:00.000 TZ=0 DST=0
	DJ = calcJD(day0, month0, year0, 0, 0,0,0,0,0,0)- 2415020;	
	//console.log("DJ=" + DJ);	
	// ************************************* Hilfsvariablen der Berechnung ************************
	K = lInt(((year0 - 1900.0 + ((DJ - J0) / 365.0)) * 12.3685) + 0.5);
	//console.log("K=" + K);	
	TN = K / 1236.85;
	TF = (K + 0.5) / 1236.85;
	T = TN;
	// Zur Unterfunktion
	// To sub function
	sechs855();
	// Hier geht es weiter
	NI = A;
	NF = B;
	NB = F;
	T = TF;
	K = K + 0.5;
	// Zur Unterfunktion
	// To the subfunction
	sechs855();
	FI = A;
	FF = B;
	FB = F;
	// Unterfunktion 6855
	// Sub function 6855
	function sechs855(){
		T2 = T * T;
		E = 29.53 * K;
		c = 166.56 + (132.87 - 0.009173 * T) * T;
		c = deg2Radians(c);
		B = 0.00058868 * K + (0.0001178 - 0.000000155 * T) * T2;
		B = B + 0.00033 * Math.sin(c ) + 0.75933;
		A = K / 12.36886;
		A1 = 359.2242 + 360.0 * (A - Math.floor(A)) - (0.0000333 + 0.00000347 * T) * T2;
		A2 = 306.0253 + 360 * ((K / 0.9330851)- Math.floor(K / 0.9330851) );
		A2 = A2 + (0.0107306 + 0.00001236 * T) * T2;
		A = K / 0.9214926;
		F = 21.2964 + 360.0 * decPart(A ) - (0.0016528 + 0.00000239 * T) * T2;
		A1 = unwindDeg(A1);
		A2 = unwindDeg(A2);
		F = unwindDeg(F);
		A1 = deg2Radians(A1);
		A2 = deg2Radians(A2);
		F = deg2Radians(F);
		DD = (0.1734 - 0.000393 * T) * Math.sin(A1 ) + 0.0021 * Math.sin(2 * A1);
		DD = DD - 0.4068 * Math.sin(A2 ) + 0.0161 * Math.sin(2 * A2) - 0.0004 * Math.sin(3 * A2 );
		DD = DD + 0.0104 * Math.sin((2 * F)) - 0.0051 * Math.sin((A1 + A2));
		DD = DD - 0.0074 * Math.sin(A1 - A2) + 0.0004 * Math.sin(2 * F + A1);
		DD = DD - 0.0004 * Math.sin(2 * F - A1 ) - 0.0006 * Math.sin(2 * F + A2) + 0.001 * Math.sin(2 * F - A2);
		DD = DD + 0.0005 * Math.sin(A1 + 2 * A2);
		E1 = Math.floor(E);
		B = B + DD + (E - E1);
		B1 = Math.floor(B);
		A = E1 + B1;
		B = B - B1;
	}
	newMoon = NI + 2415020 + NF;
	// Und Returnwert an Caller.
	// And return value to caller.
	return(newMoon);
	
}// End of newMoon()
 
 
 
  /* *********************************************************************************
	Die Funktion berechnet das Julianische Datum des Vollmondes. Der Wert 
	ist aber nicht das Datum und Uhrzeit des Vollmondes.
	Es ist die Universal Time und muss weiter berechnet werden.
					
	bergabewert:					
		dst			= Daylight Saving Time als Integer
		tz				= Timezone als Integer
		day			= Berechnungstag als Integer
		month		= Berechnungsmonat als Integer
		year			= Berechnungsjahr als Integer
		oJD			=	Julianisches Datum der Berechnung mit Tz und DST zur Kontrolle
		oJD0		=	Julianisches Datum der Berechnung 12:00 Uhr Greenwich DST=0, TZ=0
				
	Returnwert:
		fullmoon	= Julianisches Datum des UT als Number
	---------------------------------------------------------------------------------------------------
	The function calculates the Julian date of the full moon. But the value is not the 
	date and time of the full moon. It is the Universal Time and must be calculated 
	further. 
	
	Transfer value: 
	
		dst 			= Daylight Saving Time as integer 
		tz 				= Timezone as integer 
		day 			= Calculation day as integer 
		month 		= Calculation month as integer 
		year		 	= Calculation year as integer 
		oJD		 	=	 Julian date of calculation with Tz and DST for control
		oJD0	 	=	 Julian date of calculation 12:00 oclock  Greenwich DST=0, TZ=0
		
	Return value: 
		fullmoon = Julian date of UT as Number			
  *************************************************************************************** */
 function fullMoon(dst, tz, day, month, year, oJD, oJD0){
	var DST = parseInt(dst);				// bergabeparameter in richtigen  Datentypen wandeln
														// Convert transfer parameters to correct data types
	var TZ = parseInt(tz);					// Die Zeitzone - The timezone
	var DAY = parseInt(day);				// Der Tag der Berechnung - The day of the calculation
	var MONTH = parseInt(month); 	// Der Monat der Berechnung - The month of the calculation
	var YEAR = parseInt(year);			// Das Jahr der Berechnung- The year of the calculation
	var JD = 0.0;								// Das Julianische Datum des Berechnungsdatums
														// The Julian date of the calculation date
	var day0 = 0.0;								// Der Tag des Berechnungsdatums
														// The day of the calculation date
	var month0 = 0.0;							// Der Monat des Berechnungsdatums
														// The month of the calculation date
	var year0 = 0.0;							// Das Jahr des Berechnungsdatums
														// The year of the calculation date
	var J0 = 0.0;									// MJD am 0. Januar Berechnungsjahr 00:00:00.000 TZ=0 DST=0
														// MJD ist 0. Januar year of calculation 00:00:00. 000 TZ=0 DST=0
	var DJ = 0.0;								// MJD am Berechnungsdatum
														// MJD year of calculation
	var  K = 0.0;									// Ab hier Hilfsvariablen der Berechnung
														// From here auxiliary variables of the calculation
	// Interne Variablen
	// Internal variables
	var TN = 0.0; var TF = 0.0; var T= 0.0; var T2 = 0.0; var  E = 0.0;
	var c = 0.0; var B = 0.0; var A = 0.0; var A1 = 0.0; var A2 = 0.0;
	var F = 0.0; var DD = 0.0; var NI = 0.0; var NF = 0.0; var B1 = 0.0;
	var NB = 0.0; var FI = 0.0; var FF = 0.0; var FB = 0.0; var fullMoon = 0.0;
	/*
		Hier wird das Eingabedatum einmal, ber das Julianische Datum, hin-
		und her gerechnet. Dies ist notwendig, da das Eingabedatum der Berechnung
		eine Differenz zum Startdatum der Berechnung aufweisen kann. Dies wird durch die
		Parameter Sommer/Winterzeit (DST) und Zeitzone (TZ) hervorgerufen.
		Die Differenzen sind besonders relevant bei Monats- und Jahresbergngen, da das
		Berechnungsdatum ins Vorjahr oder Vormonat rutschen kann.
								
		Z.B. Eingabedatum 			01.01.2020 DST=0 TZ =-12
		Berechnungsdatum 			02.02.2020
		-----------------------------------------------------------------------------------------------------------------------------
		Here the input date is once, over the Julian date,
		and reckoned back. This is necessary because the input date of the calculation
		may have a difference to the start date of the calculation. This is due to the
		Parameters summer/winter time (DST) and time zone (TZ). 
		The differences are particularly relevant in the case of monthly and annual transitions, as the
		Calculation date can slide into the previous year or month. 
								
		E. g. input date 				01. 01. 2020 DST=0 TZ=-12
		Calculation date: 				02. 02. 2020
	*/											
	// Julianisches Datum des Berechnungsdatum um 12:00:00.000 Uhr
	// mit Sommer/Winterzeit und Zeitzone berechnen ...
	// Julian date of the calculation date at 12:00:00. 000
	// calculate with summer/winter time and time zone . . .
	JD = calcJD(DAY, MONTH, YEAR, 12,  0, 0, 0, DST,  TZ);
	// Ausgabe des JD, nur zur Kontrolle.
	// Output of the JD, only for control.
	oJD.value = JD;					
	// Aktuelles Julianisches Datum mit der Funktion calcJD2Date() und dem
	// Parameter des aktuellen Julianischen Datums	(JD)	in 
	// Datumsformat als Array umwandeln ...
	//
	// Array[0] = Das Datum als Gleitkommazahl. 
	// Dezimalstellen sind die Zeit. 
	// Array[1] = Der Monat des Datums als Ganzzahl. 
	// Array[2] = Das Jahr des Datums als Ganzzahl.
	//
	// Current Julian date iwith the function calcJD2Date () and the
	// Parameters of the current Julian date (JD) in
	// Convert date format as an array . . .
	//
	// Array[0] = The date as floating point number. 
	// Decimal places are the time. 
	// Array[1] = The month of the date as an integer. 
	// Array[2] = The year of the date as an integer. 
	var jdDate = calcJD2Date(JD);
	day0 = Math.floor(jdDate[0]);
	month0 = Math.floor(jdDate[1]);
	year0 = Math.floor(jdDate[2] );			
	// Wenn das Jahr negativ wird + 1. Es muss das Jahr nur immer um
	// 1 Jahr grer werden.
	// If the year becomes negative + 1. It just always has to be around the year
	// grow 1 year.
	if(year0 < 0)year0 += 1;
							
	// Ausgabe des Berechnungsdatums, nur zur Kontrolle.
	// Output of the  calculating date, only for control.
	//oStartDate.value = LPad(Math.floor(jdDate[0]), 2, "0") +  "." +  
								 //LPad(Math.floor(jdDate[1]), 2, "0") +  "."  + 
								 //LPad(Math.floor(jdDate[2] ), 4, "0");
	// Julianisches Datum des Startdatums ohne DST und TZ, also UT 12:00 Uhr	
	// Julian date of start without DST and TZ, it's UT 12:00 a clock high noon						
	oJD0.value = calcJD(day0, month0, year0, 12, 0, 0, 0, 0, 0);
	// ---------------------------------------------------------------------------------------------------------------------------
	// JD = 2415020 = Datum 0.1.1900 12:00 Uhr oder 31.12.1899 12:00 Uhr - TZ=0 DST=0
	// MJD am 0. Januar Berechnungsjahr
	J0 = calcJD(0, 1, year0, 0, 0,0,0,0,0,0)- 2415020;
	// MJD am Berechnungsdatum 00:00:00.000 TZ=0 DST=0
	DJ = calcJD(day0, month0, year0, 0, 0,0,0,0,0,0)- 2415020;			
	// ************* Hilfsvariablen der Berechnung / Auxiliary variables of the calculation *************
	K = K = lInt(((year0 - 1900.0 + ((DJ - J0) / 365.0)) * 12.3685) + 0.5);
	TN = K / 1236.85;
	TF = (K + 0.5) / 1236.85;
	T = TN;
	// Zur Unterfunktion
	sechs855();
							
	// Hier geht es nach sechs855()weiter
	NI = A;
	NF = B;
	NB = F;
	T = TF;
	K = K + 0.5;
							
	// Zur Unterfunktion
	// To the subfunction
	sechs855();
	FI = A;
	FF = B;
	FB = F;
							
	// Unterfunktion 6855
	// Sub function 6855
	function sechs855(){
		T2 = T * T;
		E = 29.53 * K;
		c = 166.56 + (132.87 - 0.009173 * T) * T;
		c = deg2Radians(c);
		B = 0.00058868 * K + (0.0001178 - 0.000000155 * T) * T2;
		B = B + 0.00033 * Math.sin(c ) + 0.75933;
		A = K / 12.36886;
		A1 = 359.2242 + 360.0 * (A - Math.floor(A)) - (0.0000333 + 0.00000347 * T) * T2;
		A2 = 306.0253 + 360 * ((K / 0.9330851)- Math.floor(K / 0.9330851) );
		A2 = A2 + (0.0107306 + 0.00001236 * T) * T2;
		A = K / 0.9214926;
		F = 21.2964 + 360.0 * decPart(A ) - (0.0016528 + 0.00000239 * T) * T2;
		A1 = unwindDeg(A1);
		A2 = unwindDeg(A2);
		F = unwindDeg(F);
		A1 = deg2Radians(A1);
		A2 = deg2Radians(A2);
		F = deg2Radians(F);
		DD = (0.1734 - 0.000393 * T) * Math.sin(A1 ) +  0.0021 * Math.sin(2 * A1);
		DD = DD - 0.4068 * Math.sin(A2 ) +  0.0161 * Math.sin(2 * A2) -  0.0004 * Math.sin(3 * A2 );
		DD = DD + 0.0104 * Math.sin(2  *  F) -  0.0051 * Math.sin(A1 + A2);
		DD = DD - 0.0074 * Math.sin(A1 - A2) +  0.0004 * Math.sin(2 * F + A1);
		DD = DD - 0.0004 * Math.sin(2 * F - A1 ) - 0.0006  *  Math.sin(2 * F + A2) +  0.001 * Math.sin(2 * F - A2);
		DD = DD + 0.0005 * Math.sin(A1 + 2 * A2);
		E1 = Math.floor(E);
		B = B + DD + (E - E1);
		B1 = Math.floor(B);
		A = E1 + B1;
		B = B - B1;
	}
	fullMoon = FI + 2415020 + FF;
	// Und Returnwert an Caller.
	// And return value to caller.
	return(fullMoon);
	
}// End of Fullmoon()